//1080p 2bpp Tile Driver Test in FlexProp C
//Raymond Allen 2020

#include "Platform.h"  //Most of the P2 board settings here

// the 2bpp tile driver with VGA output
struct __using("1080p_TileDriver_8b.spin2") vga; //RJA:  Visual Studio will show an error here, but it's OK

// access to the cogless graphics driver inside the vga driver
struct __using("Graphics3g.spin2") gr; //RJA:  Visual Studio will show an error here, but it's OK

//GarryJ's one cog USB Keyboard/Mouse driver
struct __using("KbM_Interface_rja2f.spin2") kbm; //RJA:  Visual Studio will show an error here, but it's OK

#include <stdio.h>
#include <math.h>
#include <time.h>
#include <stdlib.h>

struct 
{
    int x;
    int y;
    int MaxX;
    int MaxY;
    int buttons;
} MouseStatus;

int DebugLong;

//Form Data
#define MaxFormText (1920/16) //make it width of screen in tiles
#define MaxFormObjects 50

typedef struct
{
    int type;
    int top;
    int left;
    int width;
    int height;
    int data;
    int status;
} FormObject;

typedef struct {
    FormObject* p[MaxFormObjects];
} FormObjectList;



typedef struct
{
    int top;
    int left;
    int width;
    int height;
    int bordercolor;
    int fillcolor;
    char text[MaxFormText];
    int textcolor;
    FormObject* pParent;
    FormObjectList* pObjectList;

} FormForm;


typedef struct
{
    int type;
    int top;
    int left;
    int width;
    int height;
    int data;
    int status;
    int textforecolor;
    int textbackcolor;
    int bordercolor;
    int highlightedbordercolor;
    int outsidecolor;
    int insidecolor;
    int hotkeyposition;
    int hotkeycolor;
    int style;
    char text[MaxFormText];
} FormButton;


/* define a struct, and a pointer-to-struct */
typedef struct
{
    int x;
    float y;
    char z[16];
}S, * S_t;




FormButton Form1Button6 = { 1,5,52 + 22,42 - 22 + 2,4,0,0,vga.bahamablue,vga.navyblue,vga.navyblue,vga.limeade,vga.silver,vga.navyblue,0,vga.red,1,"Play Wave File" };

FormObjectList Form1Objects = { {(FormObject*)&Form1Button6,(FormObject*)0} };

FormForm Form1 = { 1,2,vga.cols - 4,vga.rows - 3,vga.navyblue,vga.Gallery,"Propeller II Mixed Signal Oscilloscope\0",vga.yellow,0, &Form1Objects };

void DrawCenteredText(int top, int left, int width, int fore, int back, char* pstr)
{//Draw centered text (like form title)
    vga.setrow(top);
    vga.setforecolor(fore);
    vga.setbackcolor(back);
    //draw centered text
    vga.SetCol(left + 1 + (width - 1 - strlen(pstr)) / 2);
    vga.print_string(pstr);
}

void FormBorder(int top, int left, int height, int width, int clr)
{//Draws border of a form
    int right = left + width - 1;
    int bottom = top + height + 1;
    //top row
    vga.setboxptr(top * vga.cols + left);
    vga.boxchr2(0x0, clr);
    for (int i = 1; i <= (width - 2); i++)
    {
        vga.boxchr2(0xC, clr);
    }
    vga.boxchr2(0x8, clr);
    //center rows
    for (int i = 1; i <= height - 2; i++)
    {
        int p = (top + i) * vga.cols + left;
        vga.setboxptr(p);
        vga.boxchr2(0xA, clr);
        p += width - 1;
        vga.setboxptr(p);
        vga.boxchr2(0xB, clr);
    }
    //bottom row
    vga.setboxptr((top + height - 1) * vga.cols + left);
    vga.boxchr2(0x1, clr);
    for (int i = 1; i <= width - 2; i++)
    {
        vga.boxchr2(0xD, clr);
    }
    vga.boxchr2(0x9, clr);
}


int DrawForm(FormForm *pForm)
{
    vga.boxfill(pForm->top, pForm->left, 2, pForm->width, pForm->bordercolor);
    DrawCenteredText(pForm->top, pForm->left, pForm->width, pForm->textcolor, pForm->bordercolor, pForm->text);
    vga.boxfill(pForm->top + 3, pForm->left + 1, pForm->height - 4, pForm->width - 2, pForm->fillcolor);
    FormBorder(pForm->top + 2, pForm->left, pForm->height - 2, pForm->width, vga.MakeColorSet(pForm->fillcolor, pForm->fillcolor, pForm->bordercolor, pForm->bordercolor));
    FormObjectList* pFOL = pForm->pObjectList;

    for (int i = 0; i < MaxFormObjects; i++)
    {
        FormObject* pFO = pFOL->p[i];
        if (pFO == 0)
        {//end of list
            break;
        }
        int type = pFO->type;
        printf("type of object #%d = %d\n", i, type);
    }
    printf("End of Form object list.\n");

}

int main()
{    //set the clock using setting in Platform.h
    //Regular way doesn't currently seem to work for 297 MHz, so getting settings from VGA driver
    //_clkset(_SETFREQ, _CLOCKFREQ);
    _clkset(vga.GetClockMode(), vga.GetClockFreq());

    //check clock freq
    int cf = _clockfreq();
    printf("clock freq= %d\n", cf);

    printf("Starting Graphics...\n");
    int pCommand = gr.start();
    int pFont = gr.GetFont();

    printf("Starting Mouse...\n");
    MouseStatus.MaxX = 1920-1;
    MouseStatus.MaxY = 1080-1;
    MouseStatus.x = 100;
    MouseStatus.y = 100;
    kbm.StartMouse(&MouseStatus);

    printf("Starting VGA...\n");
    vga.start(&MouseStatus, pCommand, pFont, & DebugLong);
    //vga.main();
    //vga.test11();
    //printf("VGA Started and returns %d\n",cog);

    vga.cls(vga.white);

    DrawForm(&Form1);

    vga.setrow(10);
    vga.setcol(10);
    vga.setforecolor(vga.white);
    vga.setbackcolor(vga.blue);
    vga.print_string("Test");

    
       
    for (;;) 
    {

    } //stop here
}
